@@ -1,7 +1,7 @@
 package App::DateUtils;
 
 our $DATE = '2016-06-29'; # DATE
-our $VERSION = '0.08'; # VERSION
+our $VERSION = '0.09'; # VERSION
 
 use 5.010001;
 use strict;
@@ -44,9 +44,12 @@ $SPEC{parse_date} = {
             schema  => ['str*', in=>[
                 'DateTime::Format::Alami::EN',
                 'DateTime::Format::Alami::ID',
+                'DateTime::Format::Flexible',
+                'DateTime::Format::Flexible(de)',
+                'DateTime::Format::Flexible(es)',
                 'DateTime::Format::Natural',
             ]],
-            default => 'DateTime::Format::Natural',
+            default => 'DateTime::Format::Flexible',
             cmdline_aliases => {m=>{}},
         },
         %time_zone_arg,
@@ -74,6 +77,10 @@ sub parse_date {
         $parser = DateTime::Format::Alami::ID->new(
             ( time_zone => $args{time_zone} ) x !!(defined($args{time_zone})),
         );
+    } elsif ($mod =~ /^DateTime::Format::Flexible/) {
+        require DateTime::Format::Flexible;
+        $parser = DateTime::Format::Flexible->new(
+        );
     } elsif ($mod eq 'DateTime::Format::Natural') {
         require DateTime::Format::Natural;
         $parser = DateTime::Format::Natural->new(
@@ -96,6 +103,24 @@ sub parse_date {
             } else {
                 $rec->{is_parseable} = 0;
             }
+        } elsif ($mod =~ /^DateTime::Format::Flexible/) {
+            my $dt;
+            my %opts;
+            $opts{lang} = [$1] if $mod =~ /\((\w+)\)$/;
+            eval { $dt = $parser->parse_datetime(
+                $date,
+                %opts,
+            ) };
+            my $err = $@;
+            if (!$err) {
+                $rec->{is_parseable} = 1;
+                $rec->{as_epoch} = $dt->epoch;
+                $rec->{as_datetime_obj} = "$dt";
+            } else {
+                $err =~ s/\n/ /g;
+                $rec->{is_parseable} = 0;
+                $rec->{error_msg} = $err;
+            }
         } elsif ($mod =~ /^DateTime::Format::Natural/) {
             my $dt = $parser->parse_datetime($date);
             if ($parser->success) {
@@ -104,6 +129,7 @@ sub parse_date {
                 $rec->{as_datetime_obj} = "$dt";
             } else {
                 $rec->{is_parseable} = 0;
+                $rec->{error_msg} = $parser->error;
             }
         }
         push @res, $rec;
@@ -111,6 +137,32 @@ sub parse_date {
     [200, "OK", \@res];
 }
 
+$SPEC{parse_date_using_df_flexible} = {
+    v => 1.1,
+    summary => 'Parse date string(s) using DateTime::Format::Flexible',
+    args => {
+        %time_zone_arg,
+        %dates_arg,
+        lang => {
+            schema => ['str*', in=>[qw/de en es/]],
+            default => 'en',
+        },
+    },
+    examples => [
+        {args => {dates => ['23rd Jun']}},
+        {args => {dates => ['23 Dez'], lang=>'de'}},
+        {args => {dates => ['foo']}},
+    ],
+};
+sub parse_date_using_df_flexible {
+    my %args = @_;
+    my $lang = $args{lang};
+    my $module = 'DateTime::Format::Flexible';
+    $module .= "(de)" if $lang eq 'de';
+    $module .= "(es)" if $lang eq 'es';
+    parse_date(module=>$module, %args);
+}
+
 $SPEC{parse_date_using_df_natural} = {
     v => 1.1,
     summary => 'Parse date string(s) using DateTime::Format::Natural',
@@ -118,6 +170,13 @@ $SPEC{parse_date_using_df_natural} = {
         %time_zone_arg,
         %dates_arg,
     },
+    examples => [
+        {args => {dates => ['23rd Jun']}},
+        {args => {dates => ['foo']}},
+    ],
+    links => [
+        {summary => 'The official CLI for DateTime::Format::Natural', url=>'dateparse'},
+    ],
 };
 sub parse_date_using_df_natural {
     my %args = @_;
@@ -131,6 +190,10 @@ $SPEC{parse_date_using_df_alami_en} = {
         %time_zone_arg,
         %dates_arg,
     },
+    examples => [
+        {args => {dates => ['23 May']}},
+        {args => {dates => ['foo']}},
+    ],
 };
 sub parse_date_using_df_alami_en {
     my %args = @_;
@@ -144,6 +207,10 @@ $SPEC{parse_date_using_df_alami_id} = {
         %time_zone_arg,
         %dates_arg,
     },
+    examples => [
+        {args => {dates => ['23 Mei']}},
+        {args => {dates => ['foo']}},
+    ],
 };
 sub parse_date_using_df_alami_id {
     my %args = @_;
@@ -221,6 +288,7 @@ sub parse_duration {
                     $dtdur->nanoseconds * 1e-9;
             } else {
                 $rec->{is_parseable} = 0;
+                $rec->{error_msg} = $parser->error;
             }
         } elsif ($mod eq 'Time::Duration::Parse') {
             my $secs;
@@ -426,7 +494,7 @@ App::DateUtils - An assortment of date-/time-related CLI utilities
 
 =head1 VERSION
 
-This document describes version 0.08 of App::DateUtils (from Perl distribution App-DateUtils), released on 2016-06-29.
+This document describes version 0.09 of App::DateUtils (from Perl distribution App-DateUtils), released on 2016-06-29.
 
 =head1 SYNOPSIS
 
@@ -445,6 +513,8 @@ date/time:
 
 =item * L<parse-date-using-df-alami-id>
 
+=item * L<parse-date-using-df-flexible>
+
 =item * L<parse-date-using-df-natural>
 
 =item * L<parse-duration>
@@ -554,7 +624,11 @@ Result:
        is_parseable    => 1,
        original        => "tomorrow",
      },
-     { is_parseable => 0, original => "foo" },
+     {
+       error_msg    => "Invalid date format: foo at /home/s1/perl5/perlbrew/perls/perl-5.22.1/lib/site_perl/5.22.1/Perinci/Access.pm line 81. ",
+       is_parseable => 0,
+       original     => "foo",
+     },
    ],
    {},
  ]
@@ -569,7 +643,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<dates>* => I<array[str]>
 
-=item * B<module> => I<str> (default: "DateTime::Format::Natural")
+=item * B<module> => I<str> (default: "DateTime::Format::Flexible")
 
 =item * B<time_zone> => I<str>
 
@@ -591,6 +665,37 @@ Return value:  (any)
 
 Parse date string(s) using DateTime::Format::Alami::EN.
 
+Examples:
+
+=over
+
+=item * Example #1:
+
+ parse_date_using_df_alami_en(dates => ["23 May"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       as_datetime_obj => "2016-05-23T00:00:00",
+       as_epoch        => 1463961600,
+       is_parseable    => 1,
+       original        => "23 May",
+       pattern         => "p_dateymd",
+     },
+   ],
+   {},
+ ]
+
+=item * Example #2:
+
+ parse_date_using_df_alami_en(dates => ["foo"]); # -> [200, "OK", [{ is_parseable => 0, original => "foo" }], {}]
+
+=back
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
@@ -619,6 +724,24 @@ Return value:  (any)
 
 Parse date string(s) using DateTime::Format::Alami::ID.
 
+Examples:
+
+=over
+
+=item * Example #1:
+
+ parse_date_using_df_alami_id(dates => ["23 Mei"]);
+
+Result:
+
+ [200, "OK", [{ is_parseable => 0, original => "23 Mei" }], {}]
+
+=item * Example #2:
+
+ parse_date_using_df_alami_id(dates => ["foo"]); # -> [200, "OK", [{ is_parseable => 0, original => "foo" }], {}]
+
+=back
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
@@ -643,10 +766,150 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 parse_date_using_df_flexible(%args) -> [status, msg, result, meta]
+
+Parse date string(s) using DateTime::Format::Flexible.
+
+Examples:
+
+=over
+
+=item * Example #1:
+
+ parse_date_using_df_flexible(dates => ["23rd Jun"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       as_datetime_obj => "2016-06-23T00:00:00",
+       as_epoch        => 1466640000,
+       is_parseable    => 1,
+       original        => "23rd Jun",
+     },
+   ],
+   {},
+ ]
+
+=item * Example #2:
+
+ parse_date_using_df_flexible(dates => ["23 Dez"], lang => "de");
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       as_datetime_obj => "2016-12-23T00:00:00",
+       as_epoch        => 1482451200,
+       is_parseable    => 1,
+       original        => "23 Dez",
+     },
+   ],
+   {},
+ ]
+
+=item * Example #3:
+
+ parse_date_using_df_flexible(dates => ["foo"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       error_msg    => "Invalid date format: foo at /home/s1/perl5/perlbrew/perls/perl-5.22.1/lib/site_perl/5.22.1/Perinci/Access.pm line 81. ",
+       is_parseable => 0,
+       original     => "foo",
+     },
+   ],
+   {},
+ ]
+
+=back
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<dates>* => I<array[str]>
+
+=item * B<lang> => I<str> (default: "en")
+
+=item * B<time_zone> => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 parse_date_using_df_natural(%args) -> [status, msg, result, meta]
 
 Parse date string(s) using DateTime::Format::Natural.
 
+Examples:
+
+=over
+
+=item * Example #1:
+
+ parse_date_using_df_natural(dates => ["23rd Jun"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       as_datetime_obj => "2016-06-23T00:00:00",
+       as_epoch        => 1466640000,
+       is_parseable    => 1,
+       original        => "23rd Jun",
+     },
+   ],
+   {},
+ ]
+
+=item * Example #2:
+
+ parse_date_using_df_natural(dates => ["foo"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       error_msg    => "'foo' does not parse (perhaps you have some garbage?)",
+       is_parseable => 0,
+       original     => "foo",
+     },
+   ],
+   {},
+ ]
+
+=back
+
 This function is not exported.
 
 Arguments ('*' denotes required arguments):
@@ -820,8 +1083,8 @@ Result:
      {
        as_dtdur_obj => "P2W",
        as_secs => 1209600,
-       date1 => "2016-06-29T11:31:13",
-       date2 => "2016-07-13T11:31:13",
+       date1 => "2016-06-29T13:56:29",
+       date2 => "2016-07-13T13:56:29",
        is_parseable => 1,
        original => "for 2 weeks",
      },
@@ -840,9 +1103,9 @@ Result:
    "OK",
    [
      {
-       as_dtdur_obj => "PT11H31M13S",
-       as_secs => 41473,
-       date1 => "2016-06-29T11:31:13",
+       as_dtdur_obj => "PT13H56M29S",
+       as_secs => 50189,
+       date1 => "2016-06-29T13:56:29",
        date2 => "2016-06-29T00:00:00",
        is_parseable => 1,
        original => "from 23 Jun to 29 Jun",
@@ -853,7 +1116,22 @@ Result:
 
 =item * Example #3:
 
- parse_duration_using_df_natural(durations => ["foo"]); # -> [200, "OK", [{ is_parseable => 0, original => "foo" }], {}]
+ parse_duration_using_df_natural(durations => ["foo"]);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     {
+       error_msg    => "'foo' does not parse (perhaps you have some garbage?)",
+       is_parseable => 0,
+       original     => "foo",
+     },
+   ],
+   {},
+ ]
 
 =back
 
@@ -913,7 +1191,7 @@ Result:
    "OK",
    [
      {
-       error_msg    => "Unknown timespec: foo at lib/App/DateUtils.pm line 227. ",
+       error_msg    => "Unknown timespec: foo at lib/App/DateUtils.pm line 295. ",
        is_parseable => 0,
        original     => "foo",
      },
@@ -962,6 +1240,9 @@ feature.
 
 =head1 SEE ALSO
 
+
+L<dateparse>, The official CLI for DateTime::Format::Natural.
+
 L<App::datecalc>
 
 =head1 AUTHOR
